##        Copyright (c) 2014/15 Rob Sykes <robs@users.sourceforge.net>
##
## This program is free software: you can redistribute it and/or modify it under
## the terms of the GNU General Public License as published by the Free Software
## Foundation, either version 3 of the License, or (at your option) any later
## version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
## FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, see <http://www.gnu.org/licenses/>.

## -*- texinfo -*-
## THIS FUNCTION IS DEPRECATED; PLEASE USE FIRPM INSTEAD.
## @deftypefn  {Function File} {@var{b} =} remez (@var{n}, @var{f}, @var{a})
## @deftypefnx {Function File} {@var{b} =} remez (@var{n}, @var{f}, @var{a}, @var{w})
## @deftypefnx {Function File} {@var{b} =} remez (@var{n}, @var{f}, @var{a}, @var{w}, @var{ftype})
## @deftypefnx {Function File} {@var{b} =} remez (@var{n}, @var{f}, @var{a}, @var{w}, @var{ftype}, @var{griddensity})
## @deftypefnx {Function File} {@var{b} =} remez (@var{n}, @var{f}, @var{a}, @var{ftype})
## @deftypefnx {Function File} {@var{b} =} remez (@var{n}, @var{f}, @var{a}, @var{ftype}, @var{griddensity})
## Parks-McClellan optimal FIR filter design.
## @table @var
## @item n
## gives the order of the returned filter
## @item f
## gives frequency at the band edges [b1 e1 b2 e2 b3 e3 @dots{}]
## Frequency is in the range [0,1], with 1 being the Nyquist frequency.
## @item a
## gives amplitude at the band edges [a(b1) a(e1) a(b2) a(e2) @dots{}]
## @item w
## gives weighting applied to each band
## @item ftype
## is "bandpass", "hilbert" or "differentiator"
## @item griddensity
## determines how accurately the filter will be constructed. The minimum value
## is 16; higher numbers are slower to compute.
## @end table
## @end deftypefn

function b = remez (N, F, A, arg4, arg5, density = 16)

if (nargin < 3 || nargin > 6)
  print_usage ();
endif

if (mod (length (F), 2))
  error ("remez: F must have even length");
endif

assignedFtype = assignedDensity = 0;
W = ones (length (F)/2, 1);
ftype = 'b';

if (nargin > 3)
  if (ischar (arg4))
    ftype = arg4; assignedFtype = 1;
  else
    W = arg4; 
  endif
  if (nargin > 4)
    if (ischar (arg5) && !assignedFtype)
      ftype = arg5;
    else
      density = arg5; assignedDensity = 1;
    endif
  endif
endif

if (length(F) != length(A))
  error ("remez: F and A must have equal length");
elseif (length(F) != length(W)*2)
  error ("remez: W must be half the length of F");
elseif (!isreal(density) || ischar(density) || (nargin > 5 && assignedDensity))
  error ("remez: incorrect argument list");
endif

b = firpm (N, F, A, W, ftype, {log(max (density,1))/log(4)-2});

endfunction

%!test assert(remez (2,[0 1],[1 0]), firpm (2,[0 1],[1 0],{0}), 0)
%!test assert(remez (2,[0 1],[1 0],'b',64), firpm (2,[0 1],[1 0],{1}), 0)
%!test F=[0 .3 .4 1];
%! assert(remez (20,F,[0 0 1 1],[10 1]), firpm (20,F,[0 1],[10 10 1 1]), 0)
